import java.io.*;
import java.nio.channels.*;
import java.net.*;
import javax.net.*;
import javax.net.ssl.*;

public class SSLConnectTest
{

	final static int port = 9876;

	static boolean use_nio = true;

	static class Server extends Thread
	{

		ServerSocket createAndBindServerSocket ( )
			throws Exception
		{
			System.setProperty("javax.net.ssl.keyStore", "C:/temp/test_keystore.jks");
			System.setProperty("javax.net.ssl.keyStoreType", "jks");
			System.setProperty("javax.net.ssl.keyStorePassword", "password");
			ServerSocketFactory factory = javax.net.ssl.SSLServerSocketFactory.getDefault();
			ServerSocket sock = factory.createServerSocket(port);
			return sock;
		}

		public void process ( Socket client )
		{
			try
			{
				System.out.println("Server: Processing client socket: " + client);
				Thread.sleep(10000);
				client.close();
			}
			catch (Exception x)
			{
				//
			}
		}

		public void run ( )
		{
			try
			{
				ServerSocket sock = createAndBindServerSocket();
				System.out.println("Server: Created and bound SSL server socket");
				while (true)
				{
					Socket client = sock.accept();
					process(client);
				}
			}
			catch (Exception x)
			{
				System.err.println("Server: Exception creating server socket: " + x);
			}
			System.out.println("Server: Exiting server thread");
		}
	}

	static class HandshakeListener implements HandshakeCompletedListener
	{
		public void handshakeCompleted ( HandshakeCompletedEvent event )
		{
			System.out.println("Client: Handshake completed");
		}
	}

	static class Client
	{

		SSLSocket createConnectAndWrapSSLSocket ( )
			throws Exception
		{
			Socket sock;
			if (use_nio)
			{
				System.out.println("Client: Creating socket with java.nio");
				SocketChannel chan = SocketChannel.open();
				sock = chan.socket();
			}
			else
			{
				System.out.println("Client: Creating socket with java.net");
				sock = new Socket();
			}
			System.out.println("Client: about to connect to server socket");
			sock.connect(new InetSocketAddress("localhost", port));
	        String host = sock.getInetAddress().getHostAddress();
	        int portnum = sock.getPort();
	        SSLSocketFactory factory = (SSLSocketFactory)javax.net.ssl.SSLSocketFactory.getDefault();
	        SSLSocket ssl_socket = (SSLSocket) factory.createSocket(sock, host, portnum, false);
			System.out.println("Client: wrapped socket with ssl socket");
			ssl_socket.addHandshakeCompletedListener(new HandshakeListener());
			System.out.println("Client: added handshake listener");
	        ssl_socket.setEnabledCipherSuites(ssl_socket.getSupportedCipherSuites());
			System.out.println("Client: setEnabledCipherSuites");
			System.out.println("Client: about to start handshake");
	        ssl_socket.startHandshake();
			System.out.println("Client: handshake complete");
			return ssl_socket;
		}

		public void doit ( )
			throws Exception
		{
			System.out.println("Client: creating client SSL socket");
			SSLSocket sock = createConnectAndWrapSSLSocket();
			System.out.println("Client: exiting client thread");
		}

	}

	public static void main ( String[] args )
		throws Exception
	{
		if (args.length > 0)
			use_nio = false;
		Server serv = new Server();
		Client cli = new Client();
		serv.start();
		Thread.sleep(1000);
		cli.doit();
	}

}

